import { Request, Response, NextFunction } from 'express';
import { createUser, loginUser } from '../models/userModel';
import { checkIn, checkOut, takeBreak, returnFromBreak, getWork } from '../models/workModel';
import { ApiError } from '../utils/ApiError';
import jwt from 'jsonwebtoken';
import { clearJwtCookie, setJwtCookie } from '../middlewares/checkAuth';


const getUserWork = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { userId } = req.body;
    const work = await getWork(userId);
    if(work instanceof ApiError) {
      return next(work);
    }
    res.status(200).json(work);
  } catch {
    const error = new ApiError('Error during user creation');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const create = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { email, password, firstName, lastName } = req.body;
    const user = await createUser({
      email,
      password,
      firstName,
      lastName
    });
    if(user instanceof ApiError) {
      return next(user);
    }
    res.status(201).json(user);
  } catch {
    const error = new ApiError('Error during user creation');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const login = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { email, password } = req.body;
    const user: any = await loginUser({
      email,
      password
    });
    if(user instanceof ApiError) {
      console.log("Error in login")
      return res.status(user.statusCode).json({ error: user.message });
    }
    const payload = {
      userId: user._id
    }
    // Generate a JWT
    const token = jwt.sign(payload, process.env.JWT_SECRET as string, { expiresIn: '1d' });
    setJwtCookie(res, token);
    // Send the JWT as the response
    res.status(200).json({
      token
    });
  } catch {
    const error = new ApiError('Error during user login');
    error.statusCode = 500;
    error.status = 'fail';
    next(error)
  }
}

const checkin = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { userId } = req.body;
    const work = await checkIn(userId);
    res.status(201).json(work);
  } catch {
    const error = new ApiError('Error during user checkin');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const takeBreakHandler = async (req: Request, res: Response, next: NextFunction) => {
  console.log("takeBreakHandler")
  try {
    const { userId } = req.body;
    const work = await takeBreak(userId);
    if(work instanceof ApiError) {
      return next(work);
    }
    res.status(201).json(work);
  } catch {
    const error = new ApiError('Error during user take break');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const returnFromBreakHandler = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { userId } = req.body;
    const work = await returnFromBreak(userId);
    if(work instanceof ApiError) {
      return next(work);
    }
    res.status(201).json(work);
  } catch {
    const error = new ApiError('Error during user return from break');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const checkout = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { userId } = req.body;
    const work = await checkOut(userId);
    res.status(201).json(work);
  } catch {
    const error = new ApiError('Error during user checkout');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const logout = async (req: Request, res: Response, next: NextFunction) => {
  try {
    clearJwtCookie(res);
    res.status(200).json({ message: 'Logout successful' });
  } catch {
    const error = new ApiError('Error during user logout');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}


export {
  create,
  logout,
  login,
  checkin,
  checkout,
  takeBreakHandler,
  returnFromBreakHandler,
  getUserWork
}