import { Work } from "../schemas/workSchema";
import { ApiError } from "../utils/ApiError";

const checkIn = async (userId: string) => {
  const startTime = new Date();
  const work = await Work.findOne({ userId, endTime: null });
  if (work) {
    return work;
  }
  const newWork = new Work({ userId, startTime, endTime: null, isBreak: false, breakStartTime: null, breakEndTime: null });
  return await newWork.save();
}

const checkOut = async (userId: string) => {
  const work = await Work.findOne({ userId, endTime: null });
  if (!work) {
    const error = new ApiError('User is not checked in');
    error.statusCode = 400;
    error.status = 'fail';
    return error;
  }
  const endTime = new Date();
  return await Work.findByIdAndUpdate(work._id, { endTime }, { new: true });
}

const takeBreak = async (userId: string) => {
  try {
    const work = await Work.findOne({ userId, endTime: null });
    if (!work) {
      const error = new ApiError('Work not found');
      error.statusCode = 404;
      error.status = 'fail';
      return error;
    }

    if (work.isBreak) {
      const error = new ApiError('Already on a break');
      error.statusCode = 400;
      error.status = 'fail';
      return error;
    }

    work.isBreak = true;
    work.breakStartTime = new Date(); // Set breakStartTime to present 0
    await work.save();
    return work;
  } catch (error) {
    const apiError = new ApiError('An error occurred');
    apiError.statusCode = 500;
    apiError.status = 'error';
    return apiError;
  }
}


const returnFromBreak = async (userId: string) => {
  try {
    const work = await Work.findOne({ userId, endTime: null });
    if (!work) {
      const error = new ApiError('Work not found');
      error.statusCode = 404;
      error.status = 'fail';
      return error;
    }

    if (!work.isBreak) {
      const error = new ApiError('Not on a break');
      error.statusCode = 400;
      error.status = 'fail';
      return error;
    }

    // Update breakDuration and totalDuration
    const breakEndTime = new Date();
    const currentTimestamp = breakEndTime.getTime();
    const breakDurationInMillis = currentTimestamp - work.breakStartTime.getTime();
    const breakDurationInHours = Number((breakDurationInMillis / 3600000).toFixed(3));
    work.breakEndTime = breakEndTime;
    work.breakDuration = breakDurationInHours + Number(work.breakDuration);
    work.isBreak = false;
    work.updatedAt = new Date();

    await work.save();
    return work;
  } catch (error) {
    const apiError = new ApiError('An error occurred');
    apiError.statusCode = 500;
    apiError.status = 'error';
    return apiError;
  }
}

const getWork = async (userId: string) => {
  try {
    const work = await Work.find({ userId });
    if (!work) {
      const error = new ApiError('Work not found');
      error.statusCode = 404;
      error.status = 'fail';
      return error;
    }
    return work;
  } catch (error) {
    const apiError = new ApiError('An error occurred');
    apiError.statusCode = 500;
    apiError.status = 'error';
    return apiError;
  }
}

export {
  checkIn,
  checkOut,
  takeBreak,
  returnFromBreak,
  getWork
}